import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.*;
import java.awt.geom.*;

/**
 *
 * @author Ceud
 */
public class RadialControl extends JPanel implements MouseListener, MouseMotionListener {
    int value = 180;
    private Point lastClick = new Point(0, 0);
    int top = 0;
    int left = 0;
    int width = 141;
    int height = 141;
    private Arc2D.Double knob = new Arc2D.Double(this.left + 120 - 10/2, this.top + 70, 10, 10, 360, 360, Arc2D.OPEN);


    BasicStroke basicStroke = new BasicStroke(2.0f);

    /**
     * @param top
     * @param left
     * @param width
     * @param height
     */
    public RadialControl(int left, int top, int width, int height)
    {
        this.top = top;
        this.left = left;
        this.width = width;
        this.height = height;
        this.value = calculateValue();

        addMouseListener(this);
        addMouseMotionListener(this);
    }

    private int calculateValue()
    {
        //click offset from origin
        int x1 = lastClick.x;
        int y1 = lastClick.y;

        //calculate polar coordinates of click
        double r = Math.sqrt(x1 * x1 + y1 * y1);
        double theta = 0d;

        if (x1 == 0 && y1 == 0) {
            theta = 0d;
        }
        else if (x1 >= 0) {
            theta = Math.toDegrees(Math.asin(y1 / r));
        }
        else if (x1 < 0) {
            double tmp = Math.asin(y1 / r);
            tmp -= tmp * 2;
            theta = Math.toDegrees(tmp + Math.PI);
        }

        return (int) theta;
    }

    @Override
    public void paint(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.clearRect(left, top, width, height);

        /*
        //draw borders
        g2d.setPaint(Color.LIGHT_GRAY);
        g2d.draw(new Line2D.Double(0, 141, 141, 141));
        g2d.draw(new Line2D.Double(141, 0, 141, 141));
        g2d.draw(new Line2D.Double(1, 1, 141, 1));
        g2d.draw(new Line2D.Double(1, 1, 1, 141));

        g2d.setPaint(Color.DARK_GRAY);
        g2d.draw(new Line2D.Double(0, 0, 0, 140));
        g2d.draw(new Line2D.Double(0, 0, 140, 0));
        g2d.draw(new Line2D.Double(0, 140, 140, 140));
        g2d.draw(new Line2D.Double(140, 0, 140, 140));
         */

        //draw outer circle
        g2d.setPaint(Color.LIGHT_GRAY);
        g2d.setStroke(basicStroke);
        g2d.draw(new Arc2D.Double(this.top + 22, this.left + 20, this.width - 40, this.height - 40, 360, 360, Arc2D.OPEN));
        g2d.setPaint(Color.DARK_GRAY);
        g2d.setStroke(basicStroke);
        g2d.draw(new Arc2D.Double(this.top + 20, this.left + 20, this.width - 40, this.height - 40, 360, 360, Arc2D.OPEN));

        //draw slider knob
        g2d.setPaint(Color.RED);
        g2d.fill(knob);

        //draw angle value in centre of control
        g2d.drawString(normaliseDegrees(this.value + 90) + "", left + ((width +1) / 2) - 10, top + ((height +1) / 2) + 5);
    }

    /**
     * @param degree Value in degrees to be normalised (any int value, positive or negative)
     * @return Normalised value in the range of 0 to 359
     */
    public static int normaliseDegrees(int degree) {
        while (degree < 0)
            degree +=360;

        while (degree >= 360)
            degree -=360;

        return degree;
    }

    private void updateControl(Point e)
    {
        //check if click is in bounds of canvas
        if (e.getX() >= this.left && e.getX() < this.left + this.width
            && e.getY() >= this.top && e.getY() < this.top + this.height) {

            int ox = (this.width + 1) / 2;
            int oy = (this.height + 1) / 2;

            this.lastClick.x = (int) e.getX() - ox;
            this.lastClick.y = (int) e.getY() - oy;
            this.value = normaliseDegrees(calculateValue());

            //slider knob placement
            knob.x = (50) * Math.cos(Math.toRadians(this.value)) + ox - 5;
            knob.y = (50) * Math.sin(Math.toRadians(this.value)) + oy - 5;

            repaint();
        }
    }

    public void mouseClicked(MouseEvent e) {
        updateControl(new Point(e.getX(), e.getY()));
    }

    public void mousePressed(MouseEvent e) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void mouseReleased(MouseEvent e) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void mouseEntered(MouseEvent e) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void mouseExited(MouseEvent e) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    public void mouseDragged(MouseEvent e) {
        updateControl(new Point(e.getX(), e.getY()));
    }

    public void mouseMoved(MouseEvent e) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }
}